---------------------------------------------------------------------------------
-- Create Procedure LOAD_RFF_REJECTS
-----------------------------------------------------------------------
call sysproc.set_routine_opts('ISOLATION UR') @



CREATE PROCEDURE LOAD_RFF_REJECTS ( )
  SPECIFIC LOAD_RFF_REJECTS
  DYNAMIC RESULT SETS 1
  LANGUAGE SQL
  NOT DETERMINISTIC
  EXTERNAL ACTION
  MODIFIES SQL DATA
  OLD SAVEPOINT LEVEL
-----------------------------------------------------------------------------
-- File:  load_rff_rejects.sql
--
--
-- Description:
--			This proc processes recrods in the FRX_RRF_REJECTS table
--
-- Called By:
--			SAS Batch Job
--
-- Calls:
--			None
--
-- Notes:     
-- 
--
-- Usage:
--		call load_rff_rejects()
--
-- History:    
--      2010/12/15   leushe   created       
--           
--          
--         
--        
--	 
--
------------------------------------------------------------------------
P1:
  BEGIN
    DECLARE v_RRF_REJECT_ID 	     BIGINT DEFAULT 0;
    DECLARE v_CMX_TRAN_ID          BIGINT DEFAULT 0;
    DECLARE v_MULTI_ORG_ID         BIGINT DEFAULT 0;
    DECLARE v_RHX_CUST_KEY         BIGINT DEFAULT 0;
    DECLARE v_RHX_ACCT_KEY         BIGINT DEFAULT 0;
    DECLARE v_RHX_CARD_KEY         BIGINT DEFAULT 0;
    DECLARE v_RHX_GRP_USERID_KEY   BIGINT DEFAULT 0;
    DECLARE v_RHX_GRO_USERID_KEY   BIGINT DEFAULT 0;
    DECLARE v_RHX_ALERT_KEY        BIGINT DEFAULT 0;
    DECLARE v_RHX_QUEUE_KEY        BIGINT DEFAULT 0;
    DECLARE v_RHX_RULE_KEY         BIGINT DEFAULT 0;
    DECLARE v_TRANSACTION_ID       BIGINT DEFAULT 0;
    DECLARE v_RHX_DATE_KEY         DATE;
    DECLARE v_RHX_TIME_KEY         TIME;
    DECLARE v_RRR_ACTION_CODE      CHAR(1);
    DECLARE v_RRR_SCORE_1          SMALLINT;
    DECLARE v_RHX_TRAN_AMT         DECIMAL(19, 5);
    DECLARE v_CREATE_ALERT_IND     CHAR(1);
    DECLARE v_RHX_COUNT_EQ_1       SMALLINT;


    DECLARE at_end                 SMALLINT DEFAULT 0;
    DECLARE l_fetch_cnt		        SMALLINT DEFAULT 0;
    DECLARE v_row_count	           INTEGER DEFAULT 0;
    DECLARE l_row_count            INTEGER DEFAULT 0;
    DECLARE v_rff_rec1_count	     INTEGER DEFAULT 0;
    DECLARE v_rff_rec2_count	     INTEGER DEFAULT 0;
    DECLARE v_rff_rec3_count	     INTEGER DEFAULT 0;
    DECLARE v_sqlstate             CHAR(5); -- variable for SQLSTATE
    DECLARE v_sqlcode              INTEGER; -- variable for SQLCODE
    DECLARE v_sqlmsg               VARCHAR(30000);

    DECLARE SQLCODE                INTEGER DEFAULT 0;
    DECLARE SQLSTATE               CHAR(5) DEFAULT '00000';


    DECLARE GLOBAL TEMPORARY TABLE rff_recs1 (
	   RRF_REJECT_ID BIGINT,
	   CMX_TRAN_ID BIGINT,
	   MULTI_ORG_ID INTEGER,
	   RHX_CUST_KEY BIGINT,
	   RHX_ACCT_KEY BIGINT,
	   RHX_CARD_KEY BIGINT,
	   RHX_GRP_USERID_KEY BIGINT,
	   RHX_GRO_USERID_KEY BIGINT,
	   TRANSACTION_ID BIGINT,
	   RHX_DATE_KEY DATE,
	   RHX_TIME_KEY TIME,
	   RRR_ACTION_CODE CHARACTER(1),
	   RRR_SCORE_1 SMALLINT,
	   RHX_TRAN_AMT DECIMAL(19,5),
	   CREATE_ALERT_IND CHARACTER(1),
	   RHX_COUNT_EQ_1 SMALLINT,
	   CMX_CREATE_TIMESTAMP TIMESTAMP,
	   RULE_ID INTEGER,
	   PACKAGE_TIMESTAMP TIMESTAMP
	 ) ON COMMIT PRESERVE ROWS;
	
	    INSERT INTO
	      session.rff_recs1
	      SELECT
	        a.RRF_REJECT_ID,a.CMX_TRAN_ID,a.MULTI_ORG_ID,a.RHX_CUST_KEY,a.RHX_ACCT_KEY,a.RHX_CARD_KEY,a.RHX_GRP_USERID_KEY,a.RHX_GRO_USERID_KEY
	        ,a.TRANSACTION_ID,a.RHX_DATE_KEY,a.RHX_TIME_KEY,a.RRR_ACTION_CODE,a.RRR_SCORE_1,a.RHX_TRAN_AMT,'0' AS CREATE_ALERT_IND,a.RHX_COUNT_EQ_1
	        ,a.CMX_CREATE_TIMESTAMP,b.rule_id,b.package_timestamp
	      FROM
	          FRX_RRF_REJECTS AS a
	        LEFT JOIN
	          FRH_BUILD_DIM AS b
	        ON a.BUILD_ID = b.BUILD_ID AND
	           a.SLOT_ID = b.SLOT_ID;
	select count(*) into v_rff_rec1_count from session.rff_recs1;
	
	DECLARE GLOBAL TEMPORARY TABLE rff_recs2 (
	   RRF_REJECT_ID BIGINT,
	   CMX_TRAN_ID BIGINT,
	   MULTI_ORG_ID INTEGER,
	   RHX_CUST_KEY BIGINT,
	   RHX_ACCT_KEY BIGINT,
	   RHX_CARD_KEY BIGINT,
	   RHX_GRP_USERID_KEY BIGINT,
	   RHX_GRO_USERID_KEY BIGINT,
	   TRANSACTION_ID BIGINT,
	   RHX_DATE_KEY DATE,
	   RHX_TIME_KEY TIME,
	   RRR_ACTION_CODE CHARACTER(1),
	   RRR_SCORE_1 SMALLINT,
	   RHX_TRAN_AMT DECIMAL(19,5),
	   CREATE_ALERT_IND CHARACTER(1),
	   RHX_COUNT_EQ_1 SMALLINT,
	   CMX_CREATE_TIMESTAMP TIMESTAMP,
	   RHX_RULE_KEY BIGINT,
	   RULE_TYPE_ID BIGINT,
	   ALERT_TYPE CHARACTER(1),
	   RHX_ALERT_VALUE_KEY BIGINT,
	   QUEUE_ID BIGINT,
	   tsdiff BIGINT
	 ) ON COMMIT PRESERVE ROWS;
	
	    INSERT INTO
	      session.rff_recs2
	      SELECT
	        a.RRF_REJECT_ID,a.CMX_TRAN_ID,a.MULTI_ORG_ID,a.RHX_CUST_KEY,a.RHX_ACCT_KEY,a.RHX_CARD_KEY,a.RHX_GRP_USERID_KEY,a.RHX_GRO_USERID_KEY
	            ,a.TRANSACTION_ID,a.RHX_DATE_KEY,a.RHX_TIME_KEY,a.RRR_ACTION_CODE,a.RRR_SCORE_1,a.RHX_TRAN_AMT,a.CREATE_ALERT_IND,a.RHX_COUNT_EQ_1
	            ,a.CMX_CREATE_TIMESTAMP,b.RHX_RULE_KEY,b.RULE_TYPE_ID,b.ALERT_TYPE       ,
	        CASE b.ALERT_TYPE
	          WHEN 'A' THEN RHX_ACCT_KEY
	          WHEN 'C' THEN RHX_CARD_KEY
	          WHEN 'X' THEN RHX_CUST_KEY
	          WHEN 'P' THEN RHX_GRP_USERID_KEY
	          WHEN 'O' THEN RHX_GRO_USERID_KEY
	          ELSE -1
	        END AS RHX_ALERT_VALUE_KEY
	       ,b.QUEUE_ID
	       ,timestampdiff(2,CHAR(a.PACKAGE_TIMESTAMP - b.CHANGE_BEGIN_TIMESTAMP)) as tsdiff
	      FROM
	        session.rff_recs1 AS a
	       ,FRH_RULE_DIM AS b
          where a.rule_id=b.rule_id AND
          timestampdiff(2,CHAR(a.PACKAGE_TIMESTAMP - b.CHANGE_BEGIN_TIMESTAMP)) >= 0;

	/* KEEP RULE RECORD CLOSEST TO PACKAGE CREATION */
		DELETE FROM  
			(SELECT row_number()          
				OVER(PARTITION BY RRF_REJECT_ID               
				ORDER BY RRF_REJECT_ID, TSDIFF) AS rn               
				FROM SESSION.RFF_RECS2)    
				WHERE rn > 1 ;

    DECLARE GLOBAL TEMPORARY TABLE reject_ids (
      CMX_TRAN_ID    BIGINT,
      TRANSACTION_ID BIGINT
    ) ON COMMIT PRESERVE ROWS;
   
       INSERT INTO
         session.reject_ids
         SELECT
           DISTINCT
           CMX_TRAN_ID,
           TRANSACTION_ID
         FROM session.rff_recs1;
	
	DROP TABLE session.rff_recs1;
	select count(*) into v_rff_rec2_count from session.rff_recs2;
	
	DECLARE GLOBAL TEMPORARY TABLE rff_recs3 (
	   RRF_REJECT_ID BIGINT,
	   CMX_TRAN_ID BIGINT,
	   MULTI_ORG_ID INTEGER,
	   RHX_CUST_KEY BIGINT,
	   RHX_ACCT_KEY BIGINT,
	   RHX_CARD_KEY BIGINT,
	   RHX_GRP_USERID_KEY BIGINT,
	   RHX_GRO_USERID_KEY BIGINT,
	   TRANSACTION_ID BIGINT,
	   RHX_DATE_KEY DATE,
	   RHX_TIME_KEY TIME,
	   RRR_ACTION_CODE CHARACTER(1),
	   RRR_SCORE_1 SMALLINT,
	   RHX_TRAN_AMT DECIMAL(19,5),
	   CREATE_ALERT_IND CHARACTER(1),
	   RHX_COUNT_EQ_1 SMALLINT,
	   RHX_RULE_KEY BIGINT,
	   RHX_ALERT_KEY BIGINT,
	   RHX_QUEUE_KEY BIGINT
 	) ON COMMIT PRESERVE ROWS;

	    INSERT INTO
	      session.rff_recs3
		 WITH alerts 
				AS (SELECT
				x.rhx_alert_key ,
				x.rhx_alert_value_key
				FROM frh_alert_dim as x
				WHERE x.rhx_alert_key=(SELECT max(rhx_alert_key) FROM frh_alert_dim as z
												 WHERE x.rhx_alert_value_key=z.rhx_alert_value_key
												 GROUP BY x.rhx_alert_value_key)
					)
	      SELECT
	        a.RRF_REJECT_ID,a.CMX_TRAN_ID,a.MULTI_ORG_ID,a.RHX_CUST_KEY,a.RHX_ACCT_KEY,a.RHX_CARD_KEY,a.RHX_GRP_USERID_KEY,a.RHX_GRO_USERID_KEY
	            ,a.TRANSACTION_ID,a.RHX_DATE_KEY,a.RHX_TIME_KEY,a.RRR_ACTION_CODE,a.RRR_SCORE_1,a.RHX_TRAN_AMT,a.CREATE_ALERT_IND,a.RHX_COUNT_EQ_1,a.RHX_RULE_KEY
		       ,CASE a.RULE_TYPE_ID
		          WHEN 203 THEN
		            coalesce(RHX_ALERT_KEY,-1)
		          ELSE
		            -1
		        END AS RHX_ALERT_KEY
		       ,CASE a.RULE_TYPE_ID
		          WHEN 203 THEN
		            coalesce(RHX_QUEUE_KEY,-1)
		          ELSE
		            -1
		        END AS RHX_QUEUE_KEY
		        --,timestampdiff(2,CHAR(a.CMX_CREATE_TIMESTAMP - b.CHANGE_BEGIN_TIMESTAMP)) as tsdiff
	      FROM
	        session.rff_recs2 AS a
			 
			 left join FRH_QUEUE_DIM as b
           on a.queue_id=b.queue_id AND
           timestampdiff(2,CHAR(a.CMX_CREATE_TIMESTAMP - b.CHANGE_BEGIN_TIMESTAMP)) >= 0
			  
	        LEFT JOIN alerts AS c
	          ON a.rhx_alert_value_key = c.rhx_alert_value_key ;
	
	    DROP TABLE session.rff_recs2;


	/* make sure no dupes */

		DELETE FROM  
			(SELECT row_number()          
				OVER(PARTITION BY RRF_REJECT_ID               
				ORDER BY CMX_TRAN_ID, TRANSACTION_ID, RHX_RULE_KEY, RHX_QUEUE_KEY, RHX_ALERT_KEY DESC) AS rn               
				FROM SESSION.RFF_RECS3)    
				WHERE rn > 1;
					
	    select count(*) into v_rff_rec3_count from session.rff_recs3;
		
  P2:
    BEGIN
          DECLARE not_found CONDITION FOR SQLSTATE '02000';
          DECLARE cursor_closed CONDITION FOR SQLSTATE '24501';
      

      DECLARE c_prep CURSOR WITH HOLD FOR
          SELECT
            RRF_REJECT_ID,CMX_TRAN_ID,MULTI_ORG_ID,RHX_CUST_KEY,RHX_ACCT_KEY,RHX_CARD_KEY,RHX_GRP_USERID_KEY,RHX_GRO_USERID_KEY,RHX_ALERT_KEY,RHX_QUEUE_KEY
                ,RHX_RULE_KEY,TRANSACTION_ID,RHX_DATE_KEY,RHX_TIME_KEY,RRR_ACTION_CODE,RRR_SCORE_1,RHX_TRAN_AMT,CREATE_ALERT_IND,RHX_COUNT_EQ_1
          FROM
            session.rff_recs3;

      /* Declare cursor to return result set to calling program */
      DECLARE c_result CURSOR WITH RETURN FOR
          SELECT
            l_fetch_cnt as FETCH_CNT
           ,l_row_count as ROWS_INSERTED
           ,v_rff_rec1_count as ROWS_INSERTED_INTO_RFF_RECS1
	        ,v_rff_rec2_count as ROWS_INSERTED_INTO_RFF_RECS2
           ,v_rff_rec3_count as ROWS_INSERTED_INTO_RFF_RECS3
           ,at_end as EXIT_CODE
           ,v_sqlstate AS OUT_SQLSTATE
           ,v_sqlcode AS OUT_SQLCODE
           ,v_sqlmsg as OUT_SQLMSG
          FROM
            SYSIBM.SYSDUMMY1;
      
      DECLARE CONTINUE HANDLER for not_found
      SET at_end = 1; 
      DECLARE CONTINUE HANDLER FOR cursor_closed
      set at_end =0;

      DECLARE EXIT HANDLER FOR SQLEXCEPTION
      BEGIN
			get diagnostics exception 1 v_sqlmsg = message_text;
			SELECT SQLSTATE,SQLCODE
              INTO v_sqlstate,v_sqlcode
			FROM  SYSIBM.SYSDUMMY1;

   	 	open c_result;
      END;


  /* Initialize variables */
  VALUES (SQLSTATE, SQLCODE) INTO v_sqlstate, v_sqlcode;


      OPEN c_prep;

      set v_row_count = 0;
      set l_fetch_cnt=1;
      set l_row_count = 0;
      WHILE (at_end = 0)
      DO
        FETCH c_prep 
        INTO  v_RRF_REJECT_ID,v_cmx_tran_id,v_MULTI_ORG_ID,v_RHX_CUST_KEY,v_RHX_ACCT_KEY,v_RHX_CARD_KEY,v_RHX_GRP_USERID_KEY,v_RHX_GRO_USERID_KEY,v_RHX_ALERT_KEY,v_RHX_QUEUE_KEY
        	    ,v_RHX_RULE_KEY,v_TRANSACTION_ID,v_RHX_DATE_KEY,v_RHX_TIME_KEY,v_RRR_ACTION_CODE,v_RRR_SCORE_1,v_RHX_TRAN_AMT,v_CREATE_ALERT_IND,v_RHX_COUNT_EQ_1;
         VALUES (SQLSTATE, SQLCODE) INTO v_sqlstate, v_sqlcode;
          
        IF (at_end = 0 ) THEN
      		set l_fetch_cnt = l_fetch_cnt + 1;
      		
				merge into FRH_RULE_FIRE_FACT
				USING TABLE(values(v_cmx_tran_id,v_MULTI_ORG_ID,v_RHX_CUST_KEY,v_RHX_ACCT_KEY,v_RHX_CARD_KEY,v_RHX_ALERT_KEY,v_RHX_QUEUE_KEY
					,v_RHX_RULE_KEY,v_TRANSACTION_ID,v_RHX_DATE_KEY,v_RHX_TIME_KEY))
		      ac(CMX_TRAN_ID,MULTI_ORG_ID,RHX_CUST_KEY,RHX_ACCT_KEY,RHX_CARD_KEY,RHX_ALERT_KEY,RHX_QUEUE_KEY
				   ,RHX_RULE_KEY,TRANSACTION_ID,RHX_DATE_KEY,RHX_TIME_KEY) 
				   
		      ON (FRH_RULE_FIRE_FACT.CMX_TRAN_ID	    = ac.CMX_TRAN_ID	
				AND FRH_RULE_FIRE_FACT.MULTI_ORG_ID	    = ac.MULTI_ORG_ID	
				AND FRH_RULE_FIRE_FACT.RHX_CUST_KEY	    = ac.RHX_CUST_KEY	
				AND FRH_RULE_FIRE_FACT.RHX_ACCT_KEY	    = ac.RHX_ACCT_KEY	
				AND FRH_RULE_FIRE_FACT.RHX_CARD_KEY	    = ac.RHX_CARD_KEY	
				AND FRH_RULE_FIRE_FACT.RHX_ALERT_KEY	 = ac.RHX_ALERT_KEY
				AND FRH_RULE_FIRE_FACT.RHX_QUEUE_KEY	 = ac.RHX_QUEUE_KEY
				AND FRH_RULE_FIRE_FACT.RHX_RULE_KEY	    = ac.RHX_RULE_KEY	
				AND FRH_RULE_FIRE_FACT.TRANSACTION_ID	 = ac.TRANSACTION_Id
				AND FRH_RULE_FIRE_FACT.RHX_DATE_KEY	    = ac.RHX_DATE_KEY	
				AND FRH_RULE_FIRE_FACT.RHX_TIME_KEY	    = ac.RHX_TIME_KEY	)
			WHEN NOT MATCHED THEN
				INSERT  				
			    (CMX_TRAN_ID,MULTI_ORG_ID,RHX_CUST_KEY,RHX_ACCT_KEY,RHX_CARD_KEY,RHX_GRP_USERID_KEY,RHX_GRO_USERID_KEY,RHX_ALERT_KEY,RHX_QUEUE_KEY
				 ,RHX_RULE_KEY,TRANSACTION_ID,RHX_DATE_KEY,RHX_TIME_KEY,RRR_ACTION_CODE,RRR_SCORE_1,RHX_TRAN_AMT,CREATE_ALERT_IND,RHX_COUNT_EQ_1
				 ,RHX_CREATE_TIMESTAMP)
			   VALUES
			     ( v_cmx_tran_id,v_MULTI_ORG_ID,v_RHX_CUST_KEY,v_RHX_ACCT_KEY,v_RHX_CARD_KEY,v_RHX_GRP_USERID_KEY,v_RHX_GRO_USERID_KEY,v_RHX_ALERT_KEY,v_RHX_QUEUE_KEY
					,v_RHX_RULE_KEY,v_TRANSACTION_ID,v_RHX_DATE_KEY,v_RHX_TIME_KEY,v_RRR_ACTION_CODE,v_RRR_SCORE_1,v_RHX_TRAN_AMT,v_CREATE_ALERT_IND,v_RHX_COUNT_EQ_1
					,CURRENT TIMESTAMP - CURRENT TIMEZONE)
			WHEN MATCHED THEN
				UPDATE set 
				(  CMX_TRAN_ID,MULTI_ORG_ID,RHX_CUST_KEY,RHX_ACCT_KEY,RHX_CARD_KEY,RHX_ALERT_KEY,RHX_QUEUE_KEY
				   ,RHX_RULE_KEY,TRANSACTION_ID,RHX_DATE_KEY,RHX_TIME_KEY
				 ) = (
				    v_cmx_tran_id,v_MULTI_ORG_ID,v_RHX_CUST_KEY,v_RHX_ACCT_KEY,v_RHX_CARD_KEY,v_RHX_ALERT_KEY,v_RHX_QUEUE_KEY
					,v_RHX_RULE_KEY,v_TRANSACTION_ID,v_RHX_DATE_KEY,v_RHX_TIME_KEY)
			ELSE IGNORE;
			
			GET DIAGNOSTICS v_row_count = ROW_COUNT;
			set l_row_count = l_row_count+v_row_count;
			IF (v_row_count > 0  ) THEN
				DELETE FROM FRX_RRF_REJECTS WHERE RRF_REJECT_ID = v_RRF_REJECT_ID;
			END IF;
			COMMIT;
  		END IF;
 		END WHILE;

     
      CLOSE c_prep;
      DROP table session.rff_recs3;
      VALUES (SQLSTATE, SQLCODE) INTO v_sqlstate, v_sqlcode;


      /* Open the cursor to return result set */
      OPEN c_result;
    END P2;

    /* FIND TRANSACTIONS NOT LOADED WILL USE IN SAS */
    
    DECLARE GLOBAL TEMPORARY TABLE not_loaded_trans (
      CMX_TRAN_ID    BIGINT,
      TRANSACTION_ID BIGINT
    ) ON COMMIT PRESERVE ROWS;
   
       INSERT INTO
         session.not_loaded_trans
         SELECT
           DISTINCT
           a.CMX_TRAN_ID,
           a.TRANSACTION_ID
         FROM session.reject_ids as a
         where NOT EXISTS (SELECT CMX_TRAN_ID from FRH_RULE_FIRE_FACT  b
         				  where a.cmx_tran_id=b.cmx_tran_id and
         				        a.transaction_id=b.transaction_id);
	
	DROP TABLE session.reject_ids;    
  END P1@



